--[===========================================[
  File: automation.lua
  Purpose: This file is used by babblevoice Desktop to aid integration
  with Desktop applications - in order to streamline workflow. For
  example, call centre agents who receive calls can automate tasks
  within the business application to automaticly lookup of accounts
  based on caller id, or even auto attendant options that the caller
  took.

  For help on Lua see www.lua.org, for information regarding
  babblevoice specific functions (the bv object) please see
  https://www.babblevoice.com/docs and look for automation.

  WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
  WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING

  When babblevoice desktop upgrades, this file will be overwritten.
  If you wish to tailor this file, copy this to a new file and
  point the automation paramater in settings.lua to your new file.

  WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
  WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
--]===========================================]

--[===========================================[
  Globals: shortcut defaults. Should be set in
  settings, but in case these are our defaults.
--]===========================================]
if nil == shortcutmobile then shortcutmobile = '1' end
if nil == shortcuthome then shortcuthome = '2' end
if nil == shortcutwork then shortcutwork = '3' end
if nil == shortcutbabble then shortcutbabble = 'b' end
if nil == shortcutsearch then shortcutsearch = 'w' end
if nil == shortcuthangup then shortcuthangup = 'h' end


--[===========================================[
  Function: getconnectedphonenumber
  Purpose: Get the phone number of the call we are
  connected to. This function uses the global object which
  is stored in the onevent function.
--]===========================================]
function getconnectedphonenumber()

	if nil ~= event then
		if nil ~= event.calls then
			for callcount = 1, #event.calls do
				if "connected" == event.calls[callcount].state then
					if "inbound" == event.calls[callcount].direction then
						return event.calls[callcount].callid
					else
						return event.calls[callcount].calledid
					end
				end
			end
		end
	end

	return ""
end


--[===========================================[
  Function: getnumberfromemis
  Purpose: Gets the phone number from the active patient in
  EMIS Web.
  There has to be a better way of doing this!!
--]===========================================]
function getnumberfromemis(hwnd, location)

	bv.setclipboardtext("")
	local p = { y = 40, x = 50 }

	-- We are interested in the first child window in the EMIS window
	local childs = bv.enumchildwindows(hwnd)



	-- Convert our points to reference the Window of interest
	-- rather than the screen.
	local ptowin = bv.clienttoscreen(childs[1], p)

	-- Now press some buttons.
	bv.sendinput(bv.VK_RBUTTON, bv.INPUT_MOUSE, ptowin)
	bv.sleep(50)

	local sub = { y = ptowin.y + 77, x =  ptowin.x + 20 }
	bv.sendinput(bv.VK_LBUTTON, bv.INPUT_MOUSE, sub)

	bv.sendstring( "\t\t\t\t\t \t\n\t\n" )

	local patientdetails = bv.waitforclipboardtext(1000)

	local details = string.match( patientdetails, location .. ": ([0-9\\(\\) ]*)" )

	if nil == details then
		bv.alert( "EMIS", "No phone number found for patient " .. location:lower() )
		return ""
	end

	bv.alert( "EMIS", "Calling: " .. details )

	return details:gsub("[^0123456789]", "")
end

--[===========================================[
  Hotkey: ^w
  Purpose: A Search for the current caller ID for EMIS Web.
--]===========================================]
bv.registerhotkey("^" .. shortcutsearch, function()

	local connectedconnectednumber = getconnectedphonenumber()


	local hwnd = bv.getforegroundwindow()
	local title = bv.getwindowtext(hwnd)

	if nil ~= string.find(title, "EMIS") then
		bv.sendinput(bv.VK_F5, bv.INPUT_KEYBOARD, bv.KEYEVENTF_KEYDOWNUP)
		local command = "\t" .. connectedconnectednumber

		if "" ~= connectedconnectednumber then
			command = command .. "\t\n"
		end

		bv.sendstring( command )
	end

end )


--[===========================================[
  Hotkey: ^b
  Purpose: Show/hide the main desktop window
--]===========================================]
bv.registerhotkey("^+" .. shortcutbabble, function()
	bv.showhide()
end )


--[===========================================[
  Function: dialcontact
  Purpose: Dials a contact based on location. Get the phone
  number from the relevant record.
--]===========================================]
function dialcontact(location)

	local hwnd = bv.getforegroundwindow()
	local title = bv.getwindowtext(hwnd)

	if nil ~= string.find(title, "EMIS") then
		if nil == useemisapifordial or
			useemisapifordial ~= true or
			"work" == location then

			local numbertodial = getnumberfromemis(hwnd, location)
			if ""  ~= numbertodial then
				bv.dial(numbertodial)
			end
		else
			bv.dial( "emis://" .. string.lower( location ) )
		end
	elseif nil ~= string.find(title, "SystmOne") then
		-- Let the SystemOne plugin handle it
		-- Note, SystmOne is not a typo
		bv.dial( "sone://" .. string.lower( location ) )
	else
		-- Generic: get selected text, if not selectet
		-- the whole text
		local number = bv.getactivecontroltext()
		local selection = bv.getactivecontrolseltext()

		if selection.e > selection.s then
			number = number:sub(selection.s, selection.e)
		end


		number = number:gsub("[^0123456789]", "")
		if string.len(number) > 0 then
				bv.dial(number)
		end
	end
end

--[===========================================[
  Hotkey: ^1 (ctrl-1)
  Purpose: Dial the phone number from an app. We can use different
  techniques based on the application that has focus.
--]===========================================]
bv.registerhotkey("^" .. shortcutmobile, function()
	dialcontact("Mobile")
end )

--[===========================================[
  Hotkey: ^2 (ctrl-2)
  Purpose: Alternate dial - for home for example.
--]===========================================]
bv.registerhotkey("^" .. shortcuthome, function()
	dialcontact("Home")
end )

--[===========================================[
  Hotkey: !d (alt-d)
  Purpose: Alternate dial - for home for example.
--]===========================================]
bv.registerhotkey("^" .. shortcutwork, function()
	dialcontact("Work")
end )

--[===========================================[
  Hotkey: !d (alt-d)
  Purpose: Alternate dial - for home for example.
--]===========================================]
bv.registerhotkey("^" .. shortcuthangup, function()
	bv.hangup()
end )

--[===========================================[
  onevent register.
  Purpose: Registers callback to receive our events from bv
  the format of the objects follows the presense API which is
  documented here:
  https://www.babblevoice.com/docs/api_presence/
--]===========================================]
bv.onevent(function(ev)

	if "device" == ev.type then
		if ev.myphone ~= nil and ev.myphone == true then
			-- This will mean we can still search after a call has finished.
			if nil ~= ev.calls then
				event = ev
			end
		end
	end
end )

bv.alert("Automation", "Shortcuts, such as ctrl-" .. shortcutmobile ..
						" to dial from your application, " ..
						"ctrl-" .. shortcuthome .. " or ctrl-" .. shortcutwork .. " to dial alternate, ctrl-" .. shortcutsearch ..
						" to search for contact, ctrl-" .. shortcuthangup .. " to hangup current call " ..
						"and " ..
						"ctrl-shift-" .. shortcutbabble .. " to toggle babblevoice Desktop. Check out support " ..
						"for your application or how to add support at babblevoice Docs under automation."
		)

